@@ -1,4 +1,4 @@
-/*	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
+/*	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -826,11 +826,10 @@ create_basic_classes (void)
   /**** <scm_class_class> ****/
   SCM cs = scm_makfrom0str (SCM_CLASS_CLASS_LAYOUT
 			    + 2 * scm_vtable_offset_user);
-  SCM cl = scm_make_struct_layout (cs);
   SCM name = Intern ("<class>");
-  scm_class_class = scm_permanent_object (scm_make_vtable_vtable (cl,
-							SCM_INUM0,
-							SCM_EOL));
+  scm_class_class = scm_permanent_object (scm_make_vtable_vtable (cs,
+								  SCM_INUM0,
+								  SCM_EOL));
   SCM_SET_CLASS_FLAGS (scm_class_class, (SCM_CLASSF_GOOPS_OR_VALID
 					 | SCM_CLASSF_METACLASS));
 
@@ -1035,6 +1034,13 @@ scm_accessor_method_slot_definition (SCM obj)
   return scm_slot_ref (obj, Intern ("slot-definition"));
 }  
 
+SCM_PROC (s_sys_tag_body, "%tag-body", 1, 0, 0, scm_sys_tag_body);
+
+SCM
+scm_sys_tag_body (SCM body)
+{
+  return scm_cons (SCM_IM_LAMBDA, body);
+}
 
 /******************************************************************************
  *
@@ -1138,7 +1144,7 @@ get_slot_value (SCM class, SCM obj, SCM slotdef)
   /* Two cases here:
    *	- access is an integer (the offset of this slot in the slots vector)
    *	- otherwise (car access) is the getter function to apply
-	   */
+   */
   if (SCM_INUMP (access))
     return SCM_SLOT (obj, SCM_INUM (access));
   else
@@ -1468,8 +1474,15 @@ wrap_init (SCM class, SCM *m, int n)
   for (i = 0; i < n; i++)
     m[i] = SCM_GOOPS_UNBOUND;
 
+#ifdef SCM_STRUCT_GC_CHAIN
+  SCM_NEWCELL2 (z);
+#else
   SCM_NEWCELL (z);
+#endif
   SCM_SETCDR (z, (SCM) m);
+#ifdef SCM_STRUCT_GC_CHAIN
+  SCM_SET_STRUCT_GC_CHAIN (z, 0);
+#endif
   SCM_SETCAR (z, (scm_bits_t) SCM_STRUCT_DATA (class) | scm_tc3_cons_gloc);
 
   return z;
@@ -1577,8 +1590,6 @@ SCM_PROC (s_sys_modify_instance, "%modify-instance", 2, 0, 0, scm_sys_modify_ins
 SCM
 scm_sys_modify_instance (SCM old, SCM new)
 {
-  SCM tmp;
-
   SCM_ASSERT (SCM_NIMP (old) && SCM_INSTANCEP (old),
 	      old, SCM_ARG1, s_sys_modify_instance);
   SCM_ASSERT (SCM_NIMP (new) && SCM_INSTANCEP (new),
@@ -1589,12 +1600,14 @@ scm_sys_modify_instance (SCM old, SCM new)
    * See "Class redefinition protocol above".
    */
   SCM_REDEFER_INTS;
-  tmp = SCM_CAR (old);
-  SCM_SETCAR (old, SCM_CAR (new));
-  SCM_SETCAR (new, tmp);
-  tmp = SCM_CDR (old);
-  SCM_SETCDR (old, SCM_CDR (new));
-  SCM_SETCDR (new, tmp);
+  {
+    SCM car = SCM_CAR (old);
+    SCM cdr = SCM_CDR (old);
+    SCM_SETCAR (old, SCM_CAR (new));
+    SCM_SETCDR (old, SCM_CDR (new));
+    SCM_SETCAR (new, car);
+    SCM_SETCDR (new, cdr);
+  }
   SCM_REALLOW_INTS;
   return SCM_UNSPECIFIED;
 }
@@ -1610,9 +1623,16 @@ scm_sys_modify_class (SCM old, SCM new)
 	      new, SCM_ARG2, s_sys_modify_class);
 
   SCM_REDEFER_INTS;
-  scm_sys_modify_instance (old, new);
-  SCM_STRUCT_DATA (old)[scm_vtable_index_vtable] = old;
-  SCM_STRUCT_DATA (new)[scm_vtable_index_vtable] = new;
+  {
+    SCM car = SCM_CAR (old);
+    SCM cdr = SCM_CDR (old);
+    SCM_SETCAR (old, SCM_CAR (new));
+    SCM_SETCDR (old, SCM_CDR (new));
+    SCM_STRUCT_DATA (old)[scm_vtable_index_vtable] = old;
+    SCM_SETCAR (new, car);
+    SCM_SETCDR (new, cdr);
+    SCM_STRUCT_DATA (new)[scm_vtable_index_vtable] = new;
+  }
   SCM_REALLOW_INTS;
   return SCM_UNSPECIFIED;
 }
@@ -1759,10 +1779,15 @@ scm_sys_invalidate_method_cache_x (SCM gf)
 	SCM_SLOT (SCM_CAR (methods), scm_si_code_table) = SCM_EOL;
     }
   {
-    int n = SCM_INUM (SCM_SLOT (gf, scm_si_n_specialized));
+    SCM n = SCM_SLOT (gf, scm_si_n_specialized);
     /* The sign of n is a flag indicating rest args. */
+    
+    /* In the separately packaged GOOPS, n-specialized in the cache
+     * is n-specialized in the gf + 1.  This is a workaround for a bug
+     * in the old SCM_IM_DISPATCH code in eval.c
+     */
     SCM_SET_MCACHE_N_SPECIALIZED (SCM_ENTITY_PROCEDURE (gf),
-				  SCM_MAKINUM (n >= 0 ? n : -n));
+				  SCM_MAKINUM (SCM_INUM (n) + 1));
   }
   return SCM_UNSPECIFIED;
 }
@@ -1905,7 +1930,7 @@ scm_i_vector2list (SCM l, int len)
 }
 
 static SCM
-sort_applicable_methods(SCM method_list, int size, SCM *targs)
+sort_applicable_methods (SCM method_list, int size, SCM *targs)
 {
   int i, j, incr;
   SCM *v, vector = SCM_EOL;
@@ -1916,44 +1941,54 @@ sort_applicable_methods(SCM method_list, int size, SCM *targs)
    * consing and reorder the list in place...
    * This idea is due to David McClain <Dave_McClain@msn.com>
    */
-  if (size <= BUFFSIZE) {
-    for(i=0;  i < size; i++) {
-      buffer[i]   = SCM_CAR (method_list);
-      method_list = SCM_CDR (method_list);
+  if (size <= BUFFSIZE)
+    {
+      for (i = 0;  i < size; i++)
+	{
+	  buffer[i]   = SCM_CAR (method_list);
+	  method_list = SCM_CDR (method_list);
+	}
+      v = buffer;
+    } 
+  else
+    {
+      /* Too many elements in method_list to keep everything locally */
+      vector = scm_i_vector2list (save, size);
+      v      = SCM_VELTS (vector);
     }
-    v = buffer;
-  } 
-  else {
-    /* Too many elements in method_list to keep everything locally */
-    vector = scm_i_vector2list (save, size);
-    v      = SCM_VELTS (vector);
-  }
 
   /* Use a simple shell sort since it is generally faster than qsort on 
    * small vectors (which is probably mostly the case when we have to
    * sort a list of applicable methods).
    */
-  for (incr = size / 2; incr; incr /= 2) {
-    for (i = incr; i < size; i++) {
-      for (j = i-incr ;j >= 0; j -= incr) {
-	if (more_specificp(v[j], v[j+incr], targs)) break;
-	else {
-	  SCM tmp   = v[j+incr];
-	  v[j+incr] = v[j];
-	  v[j]	    = tmp;
+  for (incr = size / 2; incr; incr /= 2)
+    {
+      for (i = incr; i < size; i++)
+	{
+	  for (j = i - incr; j >= 0; j -= incr)
+	    {
+	      if (more_specificp (v[j], v[j+incr], targs))
+		break;
+	      else
+		{
+		  SCM tmp = v[j + incr];
+		  v[j + incr] = v[j];
+		  v[j] = tmp;
+		}
+	    }
 	}
-      }
     }
-  }
 
-  if (size <= BUFFSIZE) {
-    /* We did it in locally, so restore the original list (reordered) in-place */
-    for(i=0, method_list=save; i < size; i++, v++) {
-      SCM_CAR(method_list) = *v;
-      method_list      = SCM_CDR(method_list);
+  if (size <= BUFFSIZE)
+    {
+      /* We did it in locally, so restore the original list (reordered) in-place */
+      for (i = 0, method_list = save; i < size; i++, v++)
+	{
+	  SCM_SETCAR (method_list, *v);
+	  method_list = SCM_CDR (method_list);
+	}
+      return save;
     }
-    return save;
-  }
   /* If we are here, that's that we did it the hard way... */ 
   return scm_vector_to_list (vector);
 }
@@ -2740,8 +2775,15 @@ SCM
 scm_wrap_object (SCM class, void *data)
 {
   SCM z;
+#ifdef SCM_STRUCT_GC_CHAIN
+  SCM_NEWCELL2 (z);
+#else
   SCM_NEWCELL (z);
+#endif
   SCM_SETCDR (z, (SCM) data);
+#ifdef SCM_STRUCT_GC_CHAIN
+  SCM_SET_STRUCT_GC_CHAIN (z, 0);
+#endif
   SCM_SETCAR (z, SCM_UNPACK (SCM_CDR (class)) | scm_tc3_cons_gloc);
   return z;
 }
@@ -2884,3 +2926,9 @@ scm_init_oop_goops_goopscore_module ()
 {
   scm_register_module_xxx ("oop goops goopscore", (void *) scm_init_goops);
 }
+
+/*
+  Local Variables:
+  c-file-style: "gnu"
+  End:
+*/
