@@ -1,6 +1,6 @@
 ;;; installed-scm-file
 
-;;;; 	Copyright (C) 1999 Free Software Foundation, Inc.
+;;;; 	Copyright (C) 1999, 2001 Free Software Foundation, Inc.
 ;;;; 
 ;;;; This program is free software; you can redistribute it and/or modify
 ;;;; it under the terms of the GNU General Public License as published by
@@ -22,7 +22,5 @@
 (define-module (oop goops internal)
   :use-module (oop goops))
 
-;; Export all bindings from (oop goops)
-(module-for-each (lambda (sym var)
-		   (module-add! %module-public-interface sym var))
-		 (nested-ref the-root-module '(app modules oop goops)))
+(set-module-uses! %module-public-interface
+		  (list (nested-ref the-root-module '(app modules oop goops))))
